module lcd_driver(
	lcd_pclk,
	rst_n,
	lcd_id,
	pixel_data,
	x_com,
	y_com,
	pixel_xpos,
	pixel_ypos,
	h_disp,
	v_disp,
	lcd_de,
	lcd_hs,
	lcd_vs,
	lcd_bl,
	lcd_clk,
	lcd_rgb,
	check_flag,
	data_lin_cross);

	//---Ports declearation: generated by Robei---
	input lcd_pclk;
	input rst_n;
	input [15:0] lcd_id;
	input [23:0] pixel_data;
	input [10:0] x_com;
	input [10:0] y_com;
	output [10:0] pixel_xpos;
	output [10:0] pixel_ypos;
	output [10:0] h_disp;
	output [10:0] v_disp;
	output lcd_de;
	output lcd_hs;
	output lcd_vs;
	output lcd_bl;
	output lcd_clk;
	output [23:0] lcd_rgb;
	output check_flag;
	output [23:0] data_lin_cross;

	wire lcd_pclk;
	wire rst_n;
	wire [15:0] lcd_id;
	wire [23:0] pixel_data;
	wire [10:0] x_com;
	wire [10:0] y_com;
	wire [10:0] pixel_xpos;
	wire [10:0] pixel_ypos;
	reg [10:0] h_disp;
	reg [10:0] v_disp;
	wire lcd_de;
	wire lcd_hs;
	wire lcd_vs;
	wire lcd_bl;
	wire lcd_clk;
	wire [23:0] lcd_rgb;
	wire check_flag;
	reg [23:0] data_lin_cross;

	//----Code starts here: integrated by Robei-----
	
	
	//parameter define  
	// 4.3' 480*272
	parameter  H_SYNC_4342   =  11'd41;     //ͬ
	parameter  H_BACK_4342   =  11'd2;      //ʾ
	parameter  H_DISP_4342   =  11'd480;    //Ч
	parameter  H_FRONT_4342  =  11'd2;      //ʾǰ
	parameter  H_TOTAL_4342  =  11'd525;    //ɨ
	   
	parameter  V_SYNC_4342   =  11'd10;     //ͬ
	parameter  V_BACK_4342   =  11'd2;      //ʾ
	parameter  V_DISP_4342   =  11'd272;    //Ч
	parameter  V_FRONT_4342  =  11'd2;      //ʾǰ
	parameter  V_TOTAL_4342  =  11'd286;    //ɨ
	parameter  RED			 =	24'hFF0000;  //ɫ
	parameter  BLACK		 =  24'h000000;	 //ɫ
	parameter  BLUE			 =  24'h0000FF;	 //ɫ
	
	//reg define
	reg  [10:0] h_sync ;
	reg  [10:0] h_back ;
	reg  [10:0] h_total;
	reg  [10:0] v_sync ;
	reg  [10:0] v_back ;
	reg  [10:0] v_total;
	reg  [10:0] h_cnt  ;
	reg  [10:0] v_cnt  ;
	reg  [10:0] x_com_r,y_com_r;
	reg  [31:0] cnt            ;
	reg  [3:0]  z_cnt          ;
	//wire define    
	wire        lcd_en;
	wire        data_req;
	//wire check_flag;        //Ŀλ
	
	
	//*****************************************************//
	//**//                    main code
	//*****************************************************//
	
	//RGB LCD DEģʽʱгͬźҪ
	assign  lcd_hs = 1'b1;        //LCDͬź
	assign  lcd_vs = 1'b1;        //LCDͬź
	
	assign  lcd_bl = 1'b1;        //LCDź  
	assign  lcd_clk = lcd_pclk;   //LCDʱ
	assign  lcd_de = lcd_en;      //LCDЧź
	
	//ʶʹ
	assign check_flag = (( x_com_r>= pixel_xpos && x_com_r<= pixel_xpos+1 )||( y_com_r>= pixel_ypos && y_com_r<= pixel_ypos +1 )) ? 1'b1 : 1'b0;
	
	//ʹRGB888
	assign  lcd_en = ((h_cnt >= h_sync + h_back) && (h_cnt < h_sync + h_back + h_disp)
	                  && (v_cnt >= v_sync + v_back) && (v_cnt < v_sync + v_back + v_disp)) 
	                  ? 1'b1 : 1'b0;
	
	//صɫ  
	assign data_req = ((h_cnt >= h_sync + h_back - 1'b1) && (h_cnt < h_sync + h_back + h_disp - 1'b1)
	                  && (v_cnt >= v_sync + v_back) && (v_cnt < v_sync + v_back + v_disp)) 
	                  ? 1'b1 : 1'b0;
	
	//ص  
	assign pixel_xpos = data_req ? (h_cnt - (h_sync + h_back - 1'b1)) : 11'd0;
	assign pixel_ypos = data_req ? (v_cnt - (v_sync + v_back - 1'b1)) : 11'd0;
	
	//RGB888
	assign lcd_rgb = lcd_en ? pixel_data : 24'd0;
	
	always@ (posedge lcd_clk) begin
	    x_com_r <= x_com;
	    y_com_r <= y_com;
	end
	
	//гʱ
	always @(posedge lcd_pclk) begin
	 
	            h_sync  <= H_SYNC_4342; 
	            h_back  <= H_BACK_4342; 
	            h_disp  <= H_DISP_4342; 
	            h_total <= H_TOTAL_4342;
	            v_sync  <= V_SYNC_4342; 
	            v_back  <= V_BACK_4342; 
	            v_disp  <= V_DISP_4342; 
	            v_total <= V_TOTAL_4342;   
	       
	end
	always @ (posedge lcd_pclk or negedge rst_n) begin
	    if (!rst_n) begin
	        data_lin_cross <= 0;
	    end
	    else begin      //  
	        if (check_flag) begin       //
	            data_lin_cross <= RED;
	        end      
	        else begin
	            data_lin_cross <= BLUE;
	        end
	    end
	end
	//мʱӼ
	always@ (posedge lcd_pclk or negedge rst_n) begin
	    if(!rst_n) 
	        h_cnt <= 11'd0;
	    else begin
	        if(h_cnt == 11'd525 - 1'b1)
	            h_cnt <= 11'd0;          
	        else begin
	            h_cnt <= h_cnt + 1'b1;            
	        end        
	    end
	end
	
	//м
	always@ (posedge lcd_pclk or negedge rst_n) begin
	    if(!rst_n) 
	        v_cnt <= 11'd0;
	    else begin
	        if(h_cnt == 11'd525 - 1'b1) begin
	            if(v_cnt == 11'd286 - 1'b1)
	                v_cnt <= 11'd0;
	            else begin
	                v_cnt <= v_cnt + 1'b1;         
	            end
	        end
	    end    
	end
	
	
	
endmodule    //lcd_driver

